from backend.src.example_optimizer import solve_optimization
# from refiner import refine_optimization
from landmarks_manager import LandmarkManager
from structs.landmarks import Landmark
from structs.landmarktype import LandmarkType
from structs.preferences import Preferences
from fastapi import FastAPI, Query, Body

app = FastAPI()
manager = LandmarkManager()

# TODO: needs a global variable to store the landmarks accross function calls
# linked_tour = []


@app.post("/route/new")
def main1(preferences: Preferences, start: tuple[float, float], end: tuple[float, float] = None) -> str:
    '''
    Main function to call the optimizer.
    :param preferences: the preferences specified by the user as the post body
    :param start: the coordinates of the starting point as a tuple of floats (as url query parameters)
    :param end: the coordinates of the finishing point as a tuple of floats (as url query parameters)
    :return: the uuid of the first landmark in the optimized route
    '''
    if preferences is None :
        raise ValueError("Please provide preferences in the form of a 'Preference' BaseModel class.")
    if start is None:
        raise ValueError("Please provide the starting coordinates as a tuple of floats.")
    if end is None:
        end = start

    start_landmark = Landmark(name='start', type=LandmarkType(landmark_type='start'), location=(start[0], start[1]), osm_type='start', osm_id=0, attractiveness=0, must_do=True, n_tags = 0)
    end_landmark = Landmark(name='end', type=LandmarkType(landmark_type='end'), location=(end[0], end[1]), osm_type='end', osm_id=0, attractiveness=0, must_do=True, n_tags = 0)
    
    # Generate the landmarks from the start location
    landmarks, landmarks_short = LandmarkManager.get_landmark_lists(preferences=preferences, coordinates=start.location)
    print([l.name for l in landmarks_short])
    # insert start and finish to the landmarks list
    landmarks_short.insert(0, start_landmark)
    landmarks_short.append(end_landmark)
    
    # TODO infer these parameters from the preferences
    max_walking_time = 4    # hours
    detour = 30             # minutes

    # First stage optimization
    base_tour = solve_optimization(landmarks_short, max_walking_time*60, True)
    
    # Second stage optimization
    # refined_tour = refine_optimization(landmarks, base_tour, max_walking_time*60+detour, True)

    # linked_tour = ...
    # return linked_tour[0].uuid
    return base_tour[0].uuid



@app.get("/landmark/{landmark_uuid}")
def get_landmark(landmark_uuid: str) -> Landmark:
    #cherche dans linked_tour et retourne le landmark correspondant
    pass
