import yaml
from typing import List, Tuple
from geopy.distance import geodesic

from structs.landmarks import Landmark
import constants

def get_time(p1: Tuple[float, float], p2: Tuple[float, float]) -> int :
    """
    Calculate the time in minutes to travel from one location to another.

    Args:
        p1 (Tuple[float, float]): Coordinates of the starting location.
        p2 (Tuple[float, float]): Coordinates of the destination.
        detour (float): Detour factor affecting the distance.
        speed (float): Walking speed in kilometers per hour.

    Returns:
        int: Time to travel from p1 to p2 in minutes.
    """

    with constants.OPTIMIZER_PARAMETERS_PATH.open('r') as f:
        parameters = yaml.safe_load(f)
        detour_factor = parameters['detour_factor']
        average_walking_speed = parameters['average_walking_speed']

    # Compute the straight-line distance in km
    if p1 == p2 :
        return 0
    else: 
        dist = geodesic(p1, p2).kilometers

    # Consider the detour factor for average cityto deterline walking distance (in km)
    walk_dist = dist*detour_factor

    # Time to walk this distance (in minutes)
    walk_time = walk_dist/average_walking_speed*60

    return round(walk_time)




# Same as link_list but does it on a already ordered list
def link_list_simple(ordered_visit: List[Landmark])-> List[Landmark] :

    L =  []
    j = 0
    total_dist = 0
    while j < len(ordered_visit)-1 :
        elem = ordered_visit[j]
        next = ordered_visit[j+1]

        elem.next_uuid = next.uuid
        d = get_time(elem.location, next.location)
        elem.time_to_reach_next = d
        if elem.name not in ['start', 'finish'] :
            elem.must_do = True
        L.append(elem)
        j += 1
        total_dist += d

    L.append(next)
    
    return L, total_dist



# Take the most important landmarks from the list
def take_most_important(landmarks: List[Landmark], N_important) -> List[Landmark] :

    L = len(landmarks)
    L_copy = []
    L_clean = []
    scores = [0]*len(landmarks)
    names = []
    name_id = {}

    for i, elem in enumerate(landmarks) :
        if elem.name not in names :
            names.append(elem.name)
            name_id[elem.name] = [i]
            L_copy.append(elem)
        else :
            name_id[elem.name] += [i]
            scores = []
            for j in name_id[elem.name] :
                scores.append(L[j].attractiveness)
            best_id = max(range(len(scores)), key=scores.__getitem__)
            t = name_id[elem.name][best_id]
            if t == i :
                for old in L_copy :
                    if old.name == elem.name :
                        old.attractiveness = L[t].attractiveness
    
    scores = [0]*len(L_copy)
    for i, elem in enumerate(L_copy) :
        scores[i] = elem.attractiveness

    res = sorted(range(len(scores)), key = lambda sub: scores[sub])[-(N_important-L):]

    for i, elem in enumerate(L_copy) :
        if i in res :
            L_clean.append(elem)

    return L_clean